@@ -31,17 +31,30 @@
 // D-Bus service names
 #define AKONADI_DBUS_SERVER_SERVICE "org.freedesktop.Akonadi"
 #define AKONADI_DBUS_CONTROL_SERVICE "org.freedesktop.Akonadi.Control"
+#define AKONADI_DBUS_CONTROL_SERVICE_LOCK "org.freedesktop.Akonadi.Control.lock"
+
+#define AKONADI_DBUS_AGENTMANAGER_PATH "/AgentManager"
 
 // Commands
+#define AKONADI_CMD_COLLECTIONCOPY "COLCOPY"
+#define AKONADI_CMD_COLLECTIONCREATE "CREATE"
+#define AKONADI_CMD_COLLECTIONDELETE "DELETE"
+#define AKONADI_CMD_COLLECTIONMODIFY "MODIFY"
+#define AKONADI_CMD_COLLECTIONMOVE "COLMOVE"
+
 #define AKONADI_CMD_ITEMCOPY "COPY"
 #define AKONADI_CMD_ITEMCREATE "X-AKAPPEND"
 #define AKONADI_CMD_ITEMDELETE "REMOVE"
 #define AKONADI_CMD_ITEMFETCH "FETCH"
+#define AKONADI_CMD_ITEMLINK "LINK"
 #define AKONADI_CMD_ITEMMODIFY "STORE"
 #define AKONADI_CMD_ITEMMOVE "MOVE"
+#define AKONADI_CMD_ITEMUNLINK "UNLINK"
+
 #define AKONADI_CMD_UID "UID"
 #define AKONADI_CMD_RESOURCESELECT "RESSELECT"
 #define AKONADI_CMD_RID "RID"
+#define AKONADI_CMD_HRID "HRID"
 
 // Command parameters
 #define AKONADI_PARAM_FULLPAYLOAD "FULLPAYLOAD"
@@ -52,6 +65,17 @@
 #define AKONADI_PARAM_SIZE "SIZE"
 #define AKONADI_PARAM_FLAGS "FLAGS"
 #define AKONADI_PARAM_REMOTEID "REMOTEID"
+#define AKONADI_PARAM_REMOTEREVISION "REMOTEREVISION"
 #define AKONADI_PARAM_UNDIRTY "DIRTY"
+#define AKONADI_PARAM_INVALIDATECACHE "INVALIDATECACHE"
+#define AKONADI_PARAM_MIMETYPE "MIMETYPE"
+#define AKONADI_PARAM_CACHEPOLICY "CACHEPOLICY"
+#define AKONADI_PARAM_NAME "NAME"
+#define AKONADI_PARAM_RESOURCE "RESOURCE"
+#define AKONADI_PARAM_ANCESTORS "ANCESTORS"
+
+#define AKONADI_PARAM_PERSISTENTSEARCH "PERSISTENTSEARCH"
+#define AKONADI_PARAM_PERSISTENTSEARCH_QUERYLANG "QUERYLANGUAGE"
+#define AKONADI_PARAM_PERSISTENTSEARCH_QUERYSTRING "QUERYSTRING"
 
 #endif
