@@ -115,7 +115,8 @@ bool Akonadi::AkAppend::commit()
 
     // set message flags
     const Flag::List flagList = HandlerHelper::resolveFlags( flags );
-    if ( !db->appendItemFlags( item, flagList, false, col ) )
+    bool flagsChanged = false;
+    if ( !db->appendItemFlags( item, flagList, flagsChanged, false, col ) )
       return failureResponse( "Unable to append item flags." );
 
     // TODO if the mailbox is currently selected, the normal new message
