@@ -32,7 +32,9 @@
 #include "resourceinterface.h"
 #include "storage/itemqueryhelper.h"
 #include "storage/itemretrievalmanager.h"
+#include "storage/itemretrievalrequest.h"
 #include "storage/parthelper.h"
+#include "storage/transaction.h"
 
 #include <QtCore/QLocale>
 #include <QtCore/QStringList>
@@ -44,29 +46,8 @@
 
 using namespace Akonadi;
 
-static const int itemQueryIdColumn = 0;
-static const int itemQueryRevColumn = 1;
-static const int itemQueryRidColumn = 2;
-static const int itemQueryMimeTypeColumn = 3;
-static const int itemQueryResouceColumn = 4;
-static const int itemQuerySizeColumn = 5;
-static const int itemQueryDatetimeColumn = 6;
-static const int itemQueryCollectionIdColumn = 7;
-
-static const int partQueryIdColumn = 0;
-static const int partQueryNameColumn = 1;
-static const int partQueryDataColumn = 2;
-static const int partQueryVersionColumn = 3;
-static const int partQueryExternalColumn = 4;
-
-FetchHelper::FetchHelper( const Scope &scope )
-  : mScope( scope ), mUseScope( true )
-{
-  init();
-}
-
-FetchHelper::FetchHelper( const ImapSet &set )
-  : mScope( Scope::None ), mSet( set ), mUseScope( false )
+FetchHelper::FetchHelper( AkonadiConnection *connection, const Scope &scope )
+  : mConnection( connection ), mScope( scope )
 {
   init();
 }
@@ -80,12 +61,7 @@ void FetchHelper::init()
   mSizeRequested = false;
   mMTimeRequested = false;
   mExternalPayloadSupported = false;
-  mConnection = 0;
-}
-
-void FetchHelper::setConnection( AkonadiConnection *connection )
-{
-  mConnection = connection;
+  mRemoteRevisionRequested = false;
 }
 
 void FetchHelper::setStreamParser( ImapStreamParser *parser )
@@ -93,39 +69,128 @@ void FetchHelper::setStreamParser( ImapStreamParser *parser )
   mStreamParser = parser;
 }
 
+enum PartQueryColumns {
+  PartQueryPimIdColumn,
+  PartQueryNameColumn,
+  PartQueryDataColumn,
+  PartQueryExternalColumn,
+  PartQueryVersionColumn
+};
+
+QSqlQuery FetchHelper::buildPartQuery( const QStringList &partList, bool allPayload, bool allAttrs )
+{
+  ///TODO: merge with ItemQuery
+  QueryBuilder partQuery( PimItem::tableName() );
+  if ( !partList.isEmpty() || allPayload || allAttrs ) {
+    partQuery.addJoin( QueryBuilder::InnerJoin, Part::tableName(), PimItem::idFullColumnName(), Part::pimItemIdFullColumnName() );
+    partQuery.addColumn( PimItem::idFullColumnName() );
+    partQuery.addColumn( Part::nameFullColumnName() );
+    partQuery.addColumn( Part::dataFullColumnName() );
+    partQuery.addColumn( Part::externalFullColumnName() );
+
+    partQuery.addColumn( Part::versionFullColumnName() );
+
+    partQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
+
+    Query::Condition cond( Query::Or );
+    if ( !partList.isEmpty() )
+      cond.addValueCondition( Part::nameFullColumnName(), Query::In, partList );
+    if ( allPayload )
+      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( "PLD:" ) );
+    if ( allAttrs )
+      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( "ATR:" ) );
+    partQuery.addCondition( cond );
+
+    ItemQueryHelper::scopeToQuery( mScope, mConnection, partQuery );
+
+    if ( !partQuery.exec() ) {
+      throw HandlerException("Unable to list item parts");
+    }
+
+    partQuery.query().next();
+  }
+
+  return partQuery.query();
+}
+
+enum ItemQueryColumns {
+  ItemQueryPimItemIdColumn,
+  ItemQueryPimItemRidColumn,
+  ItemQueryMimeTypeColumn,
+  ItemQueryResourceColumn,
+  ItemQueryRevColumn,
+  ItemQueryRemoteRevisionColumn,
+  ItemQuerySizeColumn,
+  ItemQueryDatetimeColumn,
+  ItemQueryCollectionIdColumn
+};
+
+QSqlQuery FetchHelper::buildItemQuery()
+{
+  QueryBuilder itemQuery( PimItem::tableName() );
+
+  itemQuery.addJoin( QueryBuilder::InnerJoin, MimeType::tableName(),
+                     PimItem::mimeTypeIdFullColumnName(), MimeType::idFullColumnName() );
+  itemQuery.addJoin( QueryBuilder::InnerJoin, Collection::tableName(),
+                     PimItem::collectionIdFullColumnName(), Collection::idFullColumnName());
+  itemQuery.addJoin( QueryBuilder::InnerJoin, Resource::tableName(),
+                     Collection::resourceIdFullColumnName(), Resource::idFullColumnName() );
+
+  // make sure the columns indexes here and in the constants defined at the top of the file
+  itemQuery.addColumn( PimItem::idFullColumnName() );
+  itemQuery.addColumn( PimItem::remoteIdFullColumnName() );
+  itemQuery.addColumn( MimeType::nameFullColumnName() );
+  itemQuery.addColumn( Resource::nameFullColumnName() );
+  itemQuery.addColumn( PimItem::revFullColumnName() );
+  itemQuery.addColumn( PimItem::remoteRevisionFullColumnName() );
+  itemQuery.addColumn( PimItem::sizeFullColumnName() );
+  itemQuery.addColumn( PimItem::datetimeFullColumnName() );
+  itemQuery.addColumn( PimItem::collectionIdFullColumnName() );
+
+  itemQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
+
+  if ( mScope.scope() != Scope::Invalid )
+    ItemQueryHelper::scopeToQuery( mScope, mConnection, itemQuery );
+
+  if ( !itemQuery.exec() ) {
+    throw HandlerException("Unable to list items");
+  }
+
+  itemQuery.query().next();
+
+  return itemQuery.query();
+}
+
+enum FlagQueryColumns {
+  FlagQueryIdColumn,
+  FlagQueryNameColumn
+};
+
+QSqlQuery FetchHelper::buildFlagQuery()
+{
+  QueryBuilder flagQuery( PimItem::tableName() );
+  flagQuery.addJoin( QueryBuilder::InnerJoin, PimItemFlagRelation::tableName(),
+                     PimItem::idFullColumnName(), PimItemFlagRelation::leftFullColumnName() );
+  flagQuery.addJoin( QueryBuilder::InnerJoin, Flag::tableName(),
+                     Flag::idFullColumnName(), PimItemFlagRelation::rightFullColumnName() );
+  flagQuery.addColumn( PimItem::idFullColumnName() );
+  flagQuery.addColumn( Flag::nameFullColumnName() );
+  ItemQueryHelper::scopeToQuery( mScope, mConnection, flagQuery );
+  flagQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
+
+  if ( !flagQuery.exec() ) {
+    throw HandlerException("Unable to retrieve item flags");
+  }
+
+  flagQuery.query().next();
+
+  return flagQuery.query();
+}
+
 bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
 {
   parseCommandStream();
-
-  if ( mUseScope )
-    triggerOnDemandFetch();
-
-  buildItemQuery();
-
-  // build flag query if needed
-  QueryBuilder flagQuery;
-  if ( mRequestedParts.contains( "FLAGS" ) ) {
-    flagQuery.addTable( PimItem::tableName() );
-    flagQuery.addTable( PimItemFlagRelation::tableName() );
-    flagQuery.addTable( Flag::tableName() );
-    flagQuery.addColumn( PimItem::idFullColumnName() );
-    flagQuery.addColumn( Flag::nameFullColumnName() );
-    flagQuery.addColumnCondition( PimItem::idFullColumnName(), Query::Equals, PimItemFlagRelation::leftFullColumnName() );
-    flagQuery.addColumnCondition( Flag::idFullColumnName(), Query::Equals, PimItemFlagRelation::rightFullColumnName() );
-    if ( mUseScope )
-      ItemQueryHelper::scopeToQuery( mScope, mConnection, flagQuery );
-    else
-      ItemQueryHelper::itemSetToQuery( mSet, true, mConnection, flagQuery );
-    flagQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
-
-    if ( !flagQuery.exec() ) {
-      emit failureResponse( QLatin1String( "Unable to retrieve item flags" ) );
-      return false;
-    }
-    flagQuery.query().next();
-  }
-  const int flagQueryIdColumn = 0;
-  const int flagQueryNameColumn = 1;
+  triggerOnDemandFetch();
 
   // retrieve missing parts
   QStringList partList, payloadList;
@@ -141,61 +206,80 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       mMTimeRequested = true;
       continue;
     }
+    if ( b == "REMOTEREVISION" ) {
+      mRemoteRevisionRequested = true;
+      continue;
+    }
     partList << QString::fromLatin1( b );
     if ( b.startsWith( "PLD:" ) )
       payloadList << QString::fromLatin1( b );
   }
-  retrieveMissingPayloads( payloadList );
+
+  if ( !mCacheOnly ) {
+    // Prepare for a call to ItemRetriever::exec();
+    // From a resource perspective the only parts that can be fetched are payloads.
+    ItemRetriever retriever( mConnection );
+    retriever.setScope( mScope );
+    retriever.setRetrieveParts( payloadList );
+    retriever.setRetrieveFullPayload( mFullPayload );
+    retriever.exec(); // There we go, retrieve the missing parts from the resource.
+  }
+
+  QSqlQuery itemQuery = buildItemQuery();
 
   // build part query if needed
-  QueryBuilder partQuery;
+  QSqlQuery partQuery;
   if ( !partList.isEmpty() || mFullPayload || mAllAttrs ) {
     partQuery = buildPartQuery( partList, mFullPayload, mAllAttrs );
-    if ( !partQuery.exec() ) {
-      emit failureResponse( QLatin1String( "Unable to retrieve item parts" ) );
-      return false;
-    }
-    partQuery.query().next();
+  }
+
+  // build flag query if needed
+  QSqlQuery flagQuery;
+  if ( mRequestedParts.contains( "FLAGS" ) ) {
+    flagQuery = buildFlagQuery();
   }
 
   // build responses
   Response response;
   response.setUntagged();
-  while ( mItemQuery.query().isValid() ) {
-    const qint64 pimItemId = mItemQuery.query().value( itemQueryIdColumn ).toLongLong();
-    const int pimItemRev = mItemQuery.query().value( itemQueryRevColumn ).toInt();
+  while ( itemQuery.isValid() ) {
+    const qint64 pimItemId = itemQuery.value( ItemQueryPimItemIdColumn ).toLongLong();
+    const int pimItemRev = itemQuery.value( ItemQueryRevColumn ).toInt();
 
     QList<QByteArray> attributes;
     attributes.append( "UID " + QByteArray::number( pimItemId ) );
     attributes.append( "REV " + QByteArray::number( pimItemRev ) );
-    attributes.append( "REMOTEID " + ImapParser::quote( mItemQuery.query().value( itemQueryRidColumn ).toString().toUtf8() ) );
-    attributes.append( "MIMETYPE " + ImapParser::quote( mItemQuery.query().value( itemQueryMimeTypeColumn ).toString().toUtf8() ) );
-    Collection::Id parentCollectionId = mItemQuery.query().value( itemQueryCollectionIdColumn ).toLongLong();
+    attributes.append( "REMOTEID " + ImapParser::quote( itemQuery.value( ItemQueryPimItemRidColumn ).toString().toUtf8() ) );
+    attributes.append( "MIMETYPE " + ImapParser::quote( itemQuery.value( ItemQueryMimeTypeColumn ).toString().toUtf8() ) );
+    Collection::Id parentCollectionId = itemQuery.value( ItemQueryCollectionIdColumn ).toLongLong();
     attributes.append( "COLLECTIONID " + QByteArray::number( parentCollectionId ) );
 
     if ( mSizeRequested ) {
-      const qint64 pimItemSize = mItemQuery.query().value( itemQuerySizeColumn ).toLongLong();
+      const qint64 pimItemSize = itemQuery.value( ItemQuerySizeColumn ).toLongLong();
       attributes.append( "SIZE " + QByteArray::number( pimItemSize ) );
     }
     if ( mMTimeRequested ) {
-      const QDateTime pimItemDatetime = mItemQuery.query().value( itemQueryDatetimeColumn ).toDateTime();
+      const QDateTime pimItemDatetime = itemQuery.value( ItemQueryDatetimeColumn ).toDateTime();
       // Date time is always stored in UTC time zone by the server.
       QString datetime = QLocale::c().toString( pimItemDatetime, QLatin1String( "dd-MMM-yyyy hh:mm:ss +0000" ) );
       attributes.append( "DATETIME " + ImapParser::quote( datetime.toUtf8() ) );
     }
+    if ( mRemoteRevisionRequested ) {
+      attributes.append( "REMOTEREVISION " + ImapParser::quote( itemQuery.value( ItemQueryRemoteRevisionColumn ).toString().toUtf8() ) );
+    }
 
     if ( mRequestedParts.contains( "FLAGS" ) ) {
       QList<QByteArray> flags;
-      while ( flagQuery.query().isValid() ) {
-        const qint64 id = flagQuery.query().value( flagQueryIdColumn ).toLongLong();
+      while ( flagQuery.isValid() ) {
+        const qint64 id = flagQuery.value( FlagQueryIdColumn ).toLongLong();
         if ( id < pimItemId ) {
-          flagQuery.query().next();
+          flagQuery.next();
           continue;
         } else if ( id > pimItemId ) {
           break;
         }
-        flags << flagQuery.query().value( flagQueryNameColumn ).toString().toUtf8();
-        flagQuery.query().next();
+        flags << flagQuery.value( FlagQueryNameColumn ).toString().toUtf8();
+        flagQuery.next();
       }
       attributes.append( "FLAGS (" + ImapParser::join( flags, " " ) + ')' );
     }
@@ -203,23 +287,23 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
     if ( mAncestorDepth > 0 )
       attributes.append( HandlerHelper::ancestorsToByteArray( mAncestorDepth, ancestorsForItem( parentCollectionId ) ) );
 
-    while ( partQuery.query().isValid() ) {
-      const qint64 id = partQuery.query().value( partQueryIdColumn ).toLongLong();
+    while ( partQuery.isValid() ) {
+      const qint64 id = partQuery.value( PartQueryPimIdColumn ).toLongLong();
       if ( id < pimItemId ) {
-        partQuery.query().next();
+        partQuery.next();
         continue;
       } else if ( id > pimItemId ) {
         break;
       }
-      QByteArray partName = partQuery.query().value( partQueryNameColumn ).toString().toUtf8();
-      QByteArray part = partQuery.query().value( partQueryNameColumn ).toString().toUtf8();
-      QByteArray data = partQuery.query().value( partQueryDataColumn ).toByteArray();
-      bool partIsExternal = partQuery.query().value( partQueryExternalColumn ).toBool();
+      QByteArray partName = partQuery.value( PartQueryNameColumn ).toString().toUtf8();
+      QByteArray part = partQuery.value( PartQueryNameColumn ).toString().toUtf8();
+      QByteArray data = partQuery.value( PartQueryDataColumn ).toByteArray();
+      bool partIsExternal = partQuery.value( PartQueryExternalColumn ).toBool();
       if ( !mExternalPayloadSupported && partIsExternal ) //external payload not supported by the client, translate the data
-        data = PartHelper::translateData(id, data, partIsExternal );
-      int version = partQuery.query().value( partQueryVersionColumn ).toInt();
+        data = PartHelper::translateData( data, partIsExternal );
+      int version = partQuery.value( PartQueryVersionColumn ).toInt();
       if ( version != 0 ) { // '0' is the default, so don't send it
-        part += "[" + QByteArray::number( version ) + "]";
+        part += '[' + QByteArray::number( version ) + ']';
       }
       if (  mExternalPayloadSupported && partIsExternal ) { // external data and this is supported by the client
         part += " [FILE] ";
@@ -236,7 +320,7 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       if ( mRequestedParts.contains( partName ) || mFullPayload || mAllAttrs )
         attributes << part;
 
-      partQuery.query().next();
+      partQuery.next();
     }
 
     // IMAP protocol violation: should actually be the sequence number
@@ -246,7 +330,7 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
     response.setString( attr );
     emit responseAvailable( response );
 
-    mItemQuery.query().next();
+    itemQuery.next();
   }
 
   // update atime
@@ -257,16 +341,15 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
 
 void FetchHelper::updateItemAccessTime()
 {
-  QueryBuilder qb( QueryBuilder::Update );
-  qb.addTable( PimItem::tableName() );
+  Transaction transaction( mConnection->storageBackend() );
+  QueryBuilder qb( PimItem::tableName(), QueryBuilder::Update );
   qb.setColumnValue( PimItem::atimeColumn(), QDateTime::currentDateTime() );
-  if ( mUseScope )
-    ItemQueryHelper::scopeToQuery( mScope, mConnection, qb );
-  else
-    ItemQueryHelper::itemSetToQuery( mSet, true, mConnection, qb );
+  ItemQueryHelper::scopeToQuery( mScope, mConnection, qb );
 
   if ( !qb.exec() )
     qWarning() << "Unable to update item access time";
+  else
+    transaction.commit();
 }
 
 void FetchHelper::triggerOnDemandFetch()
@@ -288,122 +371,6 @@ void FetchHelper::triggerOnDemandFetch()
   ItemRetrievalManager::instance()->requestCollectionSync( collection );
 }
 
-QueryBuilder FetchHelper::buildPartQuery( const QStringList &partList, bool allPayload, bool allAttrs )
-{
-  QueryBuilder partQuery;
-  if ( !partList.isEmpty() || allPayload || allAttrs ) {
-    partQuery.addTable( PimItem::tableName() );
-    partQuery.addTable( Part::tableName() );
-    partQuery.addColumn( PimItem::idFullColumnName() );
-    partQuery.addColumn( Part::nameFullColumnName() );
-    partQuery.addColumn( Part::dataFullColumnName() );
-    partQuery.addColumn( Part::versionFullColumnName() );
-    partQuery.addColumn( Part::externalFullColumnName() );
-    partQuery.addColumnCondition( PimItem::idFullColumnName(), Query::Equals, Part::pimItemIdFullColumnName() );
-    Query::Condition cond( Query::Or );
-    if ( !partList.isEmpty() )
-      cond.addValueCondition( Part::nameFullColumnName(), Query::In, partList );
-    if ( allPayload )
-      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( "PLD:" ) );
-    if ( allAttrs )
-      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( "ATR:" ) );
-    partQuery.addCondition( cond );
-    if ( mUseScope )
-      ItemQueryHelper::scopeToQuery( mScope, mConnection, partQuery );
-    else
-      ItemQueryHelper::itemSetToQuery( mSet, true, mConnection, partQuery );
-    partQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
-  }
-  return partQuery;
-}
-
-void FetchHelper::buildItemQuery()
-{
-  mItemQuery.addTable( PimItem::tableName() );
-  mItemQuery.addTable( MimeType::tableName() );
-  mItemQuery.addTable( Collection::tableName() );
-  mItemQuery.addTable( Resource::tableName() );
-  // make sure the columns indexes here and in the constants defined above match
-  mItemQuery.addColumn( PimItem::idFullColumnName() );
-  mItemQuery.addColumn( PimItem::revFullColumnName() );
-  mItemQuery.addColumn( PimItem::remoteIdFullColumnName() );
-  mItemQuery.addColumn( MimeType::nameFullColumnName() );
-  mItemQuery.addColumn( Resource::nameFullColumnName() );
-  mItemQuery.addColumn( PimItem::sizeFullColumnName() );
-  mItemQuery.addColumn( PimItem::datetimeFullColumnName() );
-  mItemQuery.addColumn( PimItem::collectionIdFullColumnName() );
-  mItemQuery.addColumnCondition( PimItem::mimeTypeIdFullColumnName(), Query::Equals, MimeType::idFullColumnName() );
-  mItemQuery.addColumnCondition( PimItem::collectionIdFullColumnName(), Query::Equals, Collection::idFullColumnName() );
-  mItemQuery.addColumnCondition( Collection::resourceIdFullColumnName(), Query::Equals, Resource::idFullColumnName() );
-  if ( mUseScope )
-    ItemQueryHelper::scopeToQuery( mScope, mConnection, mItemQuery );
-  else
-    ItemQueryHelper::itemSetToQuery( mSet, true, mConnection, mItemQuery );
-  mItemQuery.addSortColumn( PimItem::idFullColumnName(), Query::Ascending );
-
-  if ( !mItemQuery.exec() )
-    throw HandlerException("Unable to list items");
-  mItemQuery.query().next();
-}
-
-void FetchHelper::retrieveMissingPayloads(const QStringList & payloadList)
-{
-  if ( mCacheOnly || (payloadList.isEmpty() && !mFullPayload) )
-    return;
-
-  // TODO: I'm sure this can be done with a single query instead of manually
-  QueryBuilder partQuery = buildPartQuery( payloadList, mFullPayload, false );
-  if ( !partQuery.exec() )
-    throw HandlerException( "Unable to retrieve item parts" );
-  partQuery.query().next();
-
-  while ( mItemQuery.query().isValid() ) {
-    const qint64 pimItemId = mItemQuery.query().value( itemQueryIdColumn ).toLongLong();
-    QStringList missingParts = payloadList;
-    while ( partQuery.query().isValid() ) {
-      const qint64 id = partQuery.query().value( partQueryIdColumn ).toLongLong();
-      if ( id < pimItemId ) {
-        partQuery.query().next();
-        continue;
-      } else if ( id > pimItemId ) {
-        break;
-      }
-      QString partName = partQuery.query().value( partQueryNameColumn ).toString();
-      if ( partName.startsWith( QLatin1String( "PLD:" ) ) ) {
-        qint64 partId = partQuery.query().value( partQueryIdColumn ).toLongLong();
-        QByteArray data = partQuery.query().value( partQueryDataColumn ).toByteArray();
-        data = PartHelper::translateData(partId, data, partQuery.query().value( partQueryExternalColumn ).toBool());
-        if ( data.isNull() ) {
-          if ( mFullPayload && !missingParts.contains( partName ) )
-            missingParts << partName;
-        } else {
-          missingParts.removeAll( partName );
-        }
-      }
-      partQuery.query().next();
-    }
-    if ( !missingParts.isEmpty() ) {
-      QStringList missingPayloadIds;
-      foreach ( const QString &s, missingParts )
-        missingPayloadIds << s.mid( 4 );
-      // TODO: how should we handle retrieval errors here? so far they have been ignored,
-      // which makes sense in some cases, do we need a command parameter for this?
-      try {
-        ItemRetrievalManager::instance()->requestItemDelivery( pimItemId,
-          mItemQuery.query().value( itemQueryRidColumn ).toString().toUtf8(),
-          mItemQuery.query().value( itemQueryMimeTypeColumn ).toString().toUtf8(),
-          mItemQuery.query().value( itemQueryResouceColumn ).toString(), missingPayloadIds );
-      } catch ( const ItemRetrieverException &e ) {
-        akError() << e.type() << ": " << e.what();
-      }
-    }
-    mItemQuery.query().next();
-  }
-
-  // rewind item query
-  mItemQuery.query().first();
-}
-
 void FetchHelper::parseCommandStream()
 {
   // macro vs. attribute list
