@@ -227,15 +227,10 @@ bool DataStore::invalidateItemCache( const PimItem &item )
   if ( !qb.exec() )
     return false;
 
-  Part::List parts = qb.result();
-  PartHelper::loadData(parts); //FIXME: not needed anymore to read back the data itself?
-
-  if ( parts.isEmpty() )
-    return true;
-
+  const Part::List parts = qb.result();
   // clear data field
-  for ( int i = 0; i < parts.count(); ++i) {
-    if ( !PartHelper::update( &(parts[ i ]), QByteArray(), 0) )
+  foreach ( Part part, parts ) {
+    if ( !PartHelper::truncate( part ) )
       return false;
   }
 
@@ -427,7 +422,8 @@ bool DataStore::appendPimItem( QList<Part> & parts,
     for(QList<Part>::iterator it = parts.begin(); it != parts.end(); ++it ) {
 
       (*it).setPimItemId( pimItem.id() );
-      (*it).setDatasize( (*it).data().size() );
+      if ( (*it).datasize() < (*it).data().size() )
+        (*it).setDatasize( (*it).data().size() );
 
 //       qDebug() << "Insert from DataStore::appendPimItem";
       if( !PartHelper::insert(&(*it)) )
