@@ -43,6 +43,12 @@ class NotificationCollector : public QObject
   Q_OBJECT
 
   public:
+    /**
+      Create a new notification collector that is not attached to
+      a DataStore and just collects notifications until you emit them manually.
+    */
+    NotificationCollector( QObject *parent = 0 );
+
     /**
       Create a new notification collector for the given DataStore @p db.
       @param db The datastore using this notification collector.
@@ -87,7 +93,7 @@ class NotificationCollector : public QObject
     void itemMoved( const PimItem &item, const Collection &collectionSrc = Collection(),
                     const Collection &collectionDest = Collection(),
                     const QString &mimeType = QString(),
-                    const QByteArray &resource = QByteArray() );
+                    const QByteArray &sourceResource = QByteArray() );
 
     /**
       Notify about a removed item.
@@ -142,6 +148,12 @@ class NotificationCollector : public QObject
     void collectionRemoved( const Collection &collection,
                             const QByteArray &resource = QByteArray() );
 
+
+    /**
+      Trigger sending of collected notifications.
+    */
+    void dispatchNotifications();
+
   Q_SIGNALS:
     void notify( const Akonadi::NotificationMessage::List &msgs );
 
