@@ -79,7 +79,8 @@ static QString sortOrderToString( Query::SortOrder order )
   return QString();
 }
 
-QueryBuilder::QueryBuilder( QueryType type ) :
+QueryBuilder::QueryBuilder( const QString& table, QueryBuilder::QueryType type ) :
+    mTable( table ),
 #ifndef QUERYBUILDER_UNITTEST
     mDatabaseType( qsqlDriverNameToDatabaseType( DataStore::self()->database().driverName() ) ),
     mQuery( DataStore::self()->database() ),
@@ -96,20 +97,38 @@ void QueryBuilder::setDatabaseType( DatabaseType type )
   mDatabaseType = type;
 }
 
-void QueryBuilder::addTable(const QString & table)
+void QueryBuilder::addJoin( JoinType joinType, const QString& table, const Query::Condition& condition )
 {
-  if ( !mTables.contains( table ) )
-    mTables << table;
+  Q_ASSERT( ( joinType == InnerJoin && ( mType == Select || mType == Update ) ) ||
+            ( joinType == LeftJoin && mType == Select ) );
+
+  if ( mJoinedTables.contains( table ) ) {
+    // InnerJoin is more restrictive than a LeftJoin, hence use that in doubt
+    mJoins[table].first = qMin( joinType, mJoins.value( table ).first );
+    mJoins[table].second.addCondition( condition );
+  } else {
+    mJoins[table] = qMakePair( joinType, condition );
+    mJoinedTables << table;
+  }
 }
 
-void QueryBuilder::addValueCondition(const QString & column, Query::CompareOperator op, const QVariant & value)
+void QueryBuilder::addJoin( JoinType joinType, const QString& table, const QString& col1, const QString& col2 )
 {
-  mRootCondition.addValueCondition( column, op, value );
+  Query::Condition condition;
+  condition.addColumnCondition( col1, Query::Equals, col2 );
+  addJoin( joinType, table, condition );
 }
 
-void QueryBuilder::addColumnCondition(const QString & column, Query::CompareOperator op, const QString & column2)
+void QueryBuilder::addValueCondition(const QString & column, Query::CompareOperator op, const QVariant & value, ConditionType type)
 {
-  mRootCondition.addColumnCondition( column, op, column2 );
+  Q_ASSERT( type == WhereCondition || ( type == HavingCondition && mType == Select ) );
+  mRootCondition[type].addValueCondition( column, op, value );
+}
+
+void QueryBuilder::addColumnCondition(const QString & column, Query::CompareOperator op, const QString & column2, ConditionType type)
+{
+  Q_ASSERT( type == WhereCondition || ( type == HavingCondition && mType == Select ) );
+  mRootCondition[type].addColumnCondition( column, op, column2 );
 }
 
 QSqlQuery& QueryBuilder::query()
@@ -121,6 +140,10 @@ bool QueryBuilder::exec()
 {
   QString statement;
 
+  // we add the ON conditions of Inner Joins in a Update query here
+  // but don't want to change the mRootCondition on each exec().
+  Query::Condition whereCondition = mRootCondition[WhereCondition];
+
   switch ( mType ) {
     case Select:
       statement += QLatin1String( "SELECT " );
@@ -129,14 +152,26 @@ bool QueryBuilder::exec()
       Q_ASSERT_X( mColumns.count() > 0, "QueryBuilder::exec()", "No columns specified" );
       statement += mColumns.join( QLatin1String( ", " ) );
       statement += QLatin1String(" FROM ");
-      Q_ASSERT_X( mTables.count() > 0, "QueryBuilder::exec()", "No tables specified" );
-      statement += mTables.join( QLatin1String( ", " ) );
+      statement += mTable;
+      foreach ( const QString& joinedTable, mJoinedTables ) {
+        const QPair< JoinType, Query::Condition >& join = mJoins.value( joinedTable );
+        switch ( join.first ) {
+          case LeftJoin:
+            statement += QLatin1String( " LEFT JOIN " );
+            break;
+          case InnerJoin:
+            statement += QLatin1String( " INNER JOIN " );
+            break;
+        }
+        statement += joinedTable;
+        statement += QLatin1String( " ON " );
+        statement += buildWhereCondition( join.second );
+      }
      break;
     case Insert:
     {
       statement += QLatin1String( "INSERT INTO " );
-      Q_ASSERT_X( mTables.count() == 1, "QueryBuilder::exec()", "Specify exactly one table" );
-      statement += mTables.first();
+      statement += mTable;
       statement += QLatin1String(" (");
       typedef QPair<QString,QVariant> StringVariantPair;
       QStringList cols, vals;
@@ -154,13 +189,21 @@ bool QueryBuilder::exec()
     }
     case Update:
     {
-      statement += QLatin1String( "UPDATE " );
-      Q_ASSERT_X( mTables.count() > 0, "QueryBuilder::exec()", "No tables specified" );
+      ///TODO: fix joined Update tables for SQLite by using subqueries
+      // put the ON condition into the WHERE part of the UPDATE query
+      foreach ( const QString& table, mJoinedTables ) {
+        QPair< JoinType, Query::Condition > join = mJoins.value( table );
+        Q_ASSERT( join.first == InnerJoin );
+        whereCondition.addCondition( join.second );
+      }
 
-      if ( mDatabaseType == MySQL ) {
-        statement += mTables.join( QLatin1String( ", " ) );
-      } else {
-         statement += mTables.at( 0 );
+      statement += QLatin1String( "UPDATE " );
+      statement += mTable;
+
+      if ( mDatabaseType == MySQL && !mJoinedTables.isEmpty() ) {
+        // for mysql we list all tables directly
+        statement += QLatin1String( ", " );
+        statement += mJoinedTables.join( QLatin1String( ", " ) );
       }
 
       statement += QLatin1String( " SET " );
@@ -175,29 +218,36 @@ bool QueryBuilder::exec()
       }
       statement += updStmts.join( QLatin1String( ", " ) );
 
-      if ( mDatabaseType != MySQL ) {
-        // FROM TABLES
-        QString part = QLatin1String( " FROM" );
-        for ( int i = 1; i <  mTables.size(); i++ ) {
-          statement += part + QLatin1String( " " ) + mTables.at( i );
-          part = QLatin1String( " JOIN " );
-        }
+      if ( mDatabaseType == PostgreSQL && !mJoinedTables.isEmpty() ) {
+        // PSQL have this syntax
+        // FROM t1 JOIN t2 JOIN ...
+        statement += QLatin1String( " FROM " );
+        statement += mJoinedTables.join( QLatin1String( " JOIN " ) );
       }
 
       break;
     }
     case Delete:
       statement += QLatin1String( "DELETE FROM " );
-      Q_ASSERT_X( mTables.count() == 1, "QueryBuilder::exec()", "Exactly one table needed" );
-      statement += mTables.first();
+      statement += mTable;
       break;
     default:
       Q_ASSERT_X( false, "QueryBuilder::exec()", "Unknown enum value" );
   }
 
-  if ( !mRootCondition.isEmpty() ) {
+  if ( !whereCondition.isEmpty() ) {
     statement += QLatin1String(" WHERE ");
-    statement += buildWhereCondition( mRootCondition );
+    statement += buildWhereCondition( whereCondition );
+  }
+
+  if ( !mGroupColumns.isEmpty() ) {
+    statement += QLatin1String(" GROUP BY ");
+    statement += mGroupColumns.join( QLatin1String( ", " ) );
+  }
+
+  if ( !mRootCondition[HavingCondition].isEmpty() ) {
+    statement += QLatin1String(" HAVING ");
+    statement += buildWhereCondition( mRootCondition[HavingCondition] );
   }
 
   if ( !mSortColumns.isEmpty() ) {
@@ -285,14 +335,16 @@ QString QueryBuilder::buildWhereCondition(const Query::Condition & cond)
   }
 }
 
-void QueryBuilder::setSubQueryMode(Query::LogicOperator op)
+void QueryBuilder::setSubQueryMode(Query::LogicOperator op, ConditionType type)
 {
-  mRootCondition.setSubQueryMode( op );
+  Q_ASSERT( type == WhereCondition || ( type == HavingCondition && mType == Select ) );
+  mRootCondition[type].setSubQueryMode( op );
 }
 
-void QueryBuilder::addCondition(const Query::Condition & condition)
+void QueryBuilder::addCondition(const Query::Condition & condition, ConditionType type)
 {
-  mRootCondition.addCondition( condition );
+  Q_ASSERT( type == WhereCondition || ( type == HavingCondition && mType == Select ) );
+  mRootCondition[type].addCondition( condition );
 }
 
 void QueryBuilder::addSortColumn(const QString & column, Query::SortOrder order )
@@ -300,6 +352,18 @@ void QueryBuilder::addSortColumn(const QString & column, Query::SortOrder order
   mSortColumns << qMakePair( column, order );
 }
 
+void QueryBuilder::addGroupColumn(const QString& column)
+{
+  Q_ASSERT( mType == Select );
+  mGroupColumns << column;
+}
+
+void QueryBuilder::addGroupColumns(const QStringList& columns)
+{
+  Q_ASSERT( mType == Select );
+  mGroupColumns += columns;
+}
+
 void QueryBuilder::setColumnValue(const QString & column, const QVariant & value)
 {
   mColumnValues << qMakePair( column, value );
